@@ -4,7 +4,7 @@ use strict;
 
 package Devel::Chitin;
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 use Scalar::Util;
 use IO::File;
@@ -15,6 +15,7 @@ use Devel::Chitin::Eval;
 use Devel::Chitin::Stack;
 use Devel::Chitin::Location;
 use Devel::Chitin::SubroutineLocation;
+use Devel::Chitin::SubroutineReturn;
 use Devel::Chitin::Exception;
 use Devel::Chitin::OpTree;
 
@@ -87,12 +88,17 @@ sub stepover {
 }
 
 sub stepout {
+    my $self = shift;
+    if (@_) {
+        my %args = @_;
+        DB::_queue_on_sub_return($args{'cb'}) if $args{'cb'};
+    }
+
     $DB::single=0;
     $DB::step_over_depth = $DB::stack_depth - 1;
     return 1;
 }
 
-# Should support running to a subname, or file+line
 sub continue {
     $DB::single=0;
     return 1;
@@ -838,6 +844,7 @@ sub sub {
     #goto &$sub if (! $ready or $in_debugger or index($sub, 'Devel::Chitin::StackTracker') == 0 or $debugger_disabled);
 
     local $Devel::Chitin::current_sub = $sub unless $in_debugger;
+    local @Devel::Chitin::on_sub_return_queue;
 
     local @AUTOLOAD_names = @AUTOLOAD_names;
     if (index($sub, '::AUTOLOAD', -10) >= 0) {
@@ -867,10 +874,23 @@ sub sub {
     my @rv;
     if (wantarray) {
         @rv = &$sub;
+        if (@Devel::Chitin::on_sub_return_queue) {
+            my $subreturn = _trigger_on_sub_return_queue(wantarray, \@rv);
+            if (Scalar::Util::reftype($subreturn->rv) eq 'ARRAY') {
+                @rv = @{ $subreturn->rv };
+            } else {
+                @rv = ( $subreturn->rv );
+            }
+        }
     } elsif (defined wantarray) {
         $rv[0] = &$sub;
+        if (@Devel::Chitin::on_sub_return_queue) {
+            my $subreturn = _trigger_on_sub_return_queue(0, $rv[0]);
+            $rv[0] = $subreturn->rv;
+        }
     } else {
         &$sub;
+        _trigger_on_sub_return_queue(undef, undef) if @Devel::Chitin::on_sub_return_queue;
     }
 
     delete $Devel::Chitin::eval_serial{$$stack_tracker} if $stack_tracker;
@@ -878,6 +898,22 @@ sub sub {
     return wantarray ? @rv : $rv[0];
 }
 
+sub _queue_on_sub_return {
+    my $cb = shift;
+    push @Devel::Chitin::on_sub_return_queue, $cb;
+}
+
+sub _trigger_on_sub_return_queue {
+    my($wantarray, @rv) = @_;
+    my $rv = $wantarray ? \@rv : $rv[0];
+
+    my $subreturn = Devel::Chitin::SubroutineReturn->new(wantarray => $wantarray,
+                                                         rv => $rv,
+                                                         map { $_ => $previous_location->$_ } qw(package subroutine filename line));
+    $_->($subreturn) foreach @Devel::Chitin::on_sub_return_queue;
+    return $subreturn;
+}
+
 sub lsub : lvalue {
     no strict 'refs';
     goto &$sub if (! $ready or index($sub, 'Devel::Chitin::StackTracker') == 0 or $debugger_disabled);
@@ -1099,6 +1135,19 @@ after that subroutine call returns.
 Continue running the debugged program until the current subroutine returns
 or until the next breakpoint, whichever comes first.
 
+=item CLIENT->stepout(cb => $subref);
+
+This form of stepout() allows registering a callback to be invoked when the
+current subroutine returns.  The callback's first argument is an instance of
+L<Devel::Chitin::SubroutineReturn>, which provides access to the location
+the function is returning from, its wantarray status, and the function's
+return value.  The C<rv> property of the SubroutineReturn object is mutable,
+and actually changes the value being returned from the function.
+
+Callbacks are invoked in the order they are queued, and a return value changed
+in this way is presented as the return value to the next callback.  The final
+callback gets the last say about the ultimate return value from the function.
+
 =item CLIENT->continue()
 
 Continue running the debugged program until the next breakpoint.
